
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "rand.h"
#include "main.h"

#include "anime.h"

//---------------------------------------------------------------------------
ROM_DATA ST_ANIME_PAT AnimePatNormal[] = {
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y },
	{ ANIME_PAT_NO_CHG_F, ""            , FACE_SPR_TILE_NO_NORMAL },
	{ ANIME_PAT_NO_IMG  , "normal.img"  , 0 },
	{ ANIME_PAT_NO_END  , ""            , 0 },
};

ROM_DATA ST_ANIME_PAT AnimePatDrop[] = {
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y },
	{ ANIME_PAT_NO_CHG_F, ""            , FACE_SPR_TILE_NO_SLEEP },
	{ ANIME_PAT_NO_IMG  , "drop_00.img" , 0 },
	{ ANIME_PAT_NO_IMG  , "drop_01.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_02.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_03.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_04.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_05.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_06.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_07.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_08.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_09.img" , 2 },
	{ ANIME_PAT_NO_IMG  , "drop_10.img" , 2 },
	{ ANIME_PAT_NO_END  , ""            , 0 },
};

ROM_DATA ST_ANIME_PAT AnimePatShake[] = {
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y },
	{ ANIME_PAT_NO_IMG  , "normal.img"  , 0 },
	{ ANIME_PAT_NO_IMG  , "shake_00.img", 2 },
	{ ANIME_PAT_NO_IMG  , "shake_01.img", 2 },
	{ ANIME_PAT_NO_IMG  , "shake_02.img", 2 },
	{ ANIME_PAT_NO_IMG  , "shake_03.img", 2 },
	{ ANIME_PAT_NO_IMG  , "shake_04.img", 2 },
	{ ANIME_PAT_NO_IMG  , "normal.img"  , 2 },
	{ ANIME_PAT_NO_END  , ""            , 0 },
};

ROM_DATA ST_ANIME_PAT AnimePatSleep1[] = {
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X-2 },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y+2 },
	{ ANIME_PAT_NO_CHG_F, ""            , FACE_SPR_TILE_NO_SLEEP },
	{ ANIME_PAT_NO_IMG  , "sleep.img"   , 0 },
	{ ANIME_PAT_NO_END  , ""            , 0 },
};

ROM_DATA ST_ANIME_PAT AnimePatSleep2[] = {
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y },
	{ ANIME_PAT_NO_CHG_F, ""            , FACE_SPR_TILE_NO_SLEEP },
	{ ANIME_PAT_NO_IMG  , "normal.img"  , 0 },
	{ ANIME_PAT_NO_END  , ""            , 0 },
};

ROM_DATA ST_ANIME_PAT AnimePatAwake[] = {
	{ ANIME_PAT_NO_CHG_F, ""            , FACE_SPR_TILE_NO_AWAKE },
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y+20 },
	{ ANIME_PAT_NO_IMG  , "awake_00.img", 0 },
	{ ANIME_PAT_NO_WAIT , ""            , 2 },
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y-15 },
	{ ANIME_PAT_NO_IMG  , "awake_01.img", 0 },
	{ ANIME_PAT_NO_WAIT , ""            , 2 },
	{ ANIME_PAT_NO_CHG_X, ""            , FACE_SPR_X },
	{ ANIME_PAT_NO_CHG_Y, ""            , FACE_SPR_Y },
	{ ANIME_PAT_NO_IMG  , "normal.img"  , 0 },
	{ ANIME_PAT_NO_WAIT , ""            , 2 },
	{ ANIME_PAT_NO_END  , ""            , 0 },
};

//---------------------------------------------------------------------------
ST_ANIME Anime;
ST_FACE  Face;

//---------------------------------------------------------------------------
EWRAM_CODE void AnimeInit()
{
	_Memset((u8*)&Anime, 0x00, sizeof(ST_ANIME));
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeSetNo(u16 no)
{
	Anime.no  = no;
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExec()
{
	switch(Anime.no)
	{
		case ANIME_NO_NORMAL:
			AnimeExecPat(AnimePatNormal);
			break;

		case ANIME_NO_DROP:
			AnimeExecPat(AnimePatDrop);
			break;

		case ANIME_NO_SHAKE:
			AnimeExecPat(AnimePatShake);
			break;

		case ANIME_NO_SLEEP1:
			AnimeExecPat(AnimePatSleep1);
			break;

		case ANIME_NO_SLEEP2:
			AnimeExecPat(AnimePatSleep2);
			break;

		case ANIME_NO_AWAKE:
			AnimeExecPat(AnimePatAwake);
			break;

		case ANIME_NO_FACE_NORMAL:
			FaceSetNo(FACE_SPR_TILE_NO_NORMAL);
			break;

		case ANIME_NO_FACE_AWAKE:
			FaceSetNo(FACE_SPR_TILE_NO_AWAKE);
			break;

		case ANIME_NO_FACE_SCARE:
			FaceSetNo(FACE_SPR_TILE_NO_SCARE);
			break;

		default:
			SystemError("[Err] AnimeSetData animeNo.", NULL);
			break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecPat(ST_ANIME_PAT* p)
{
	while(p->no != ANIME_PAT_NO_END)
	{
		switch(p->no)
		{
			case ANIME_PAT_NO_IMG:
				AnimeExecPatImg(p->name, p->var);
				break;

			case ANIME_PAT_NO_CHG_X:
				FaceSetX(p->var);
				break;

			case ANIME_PAT_NO_CHG_Y:
				FaceSetY(p->var);
				break;

			case ANIME_PAT_NO_CHG_F:
				FaceSetNo(p->var);
				break;

			case ANIME_PAT_NO_WAIT:
				AnimeExecPatWait(p->var);
				break;

			default:
				SystemError("[Err] AnimeExecPat p->no.", NULL);
				break;
		}

		p++;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecPatImg(char* pName, u16 wait)
{
	u8* pData = GBFSGetFilePointer(pName);

	if(pData == NULL)
	{
		SystemError("[Err] Image File Not Found.", pName);
	}

	AnimeExecPatWait(wait);
	CpuSet((u32)pData, (u32)VideoBuffer, (76800 / 4) | CPUSET_32BIT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void AnimeExecPatWait(u16 wait)
{
	u16 i;
	for(i=0; i<wait; i++)
	{
		WaitForVsync();
	}
}



//---------------------------------------------------------------------------
EWRAM_CODE void FaceInit()
{
	FaceInitSpr();

	_Memset((u8*)&Face, 0x00, sizeof(ST_FACE));

	FaceSetX(FACE_SPR_X);
	FaceSetY(FACE_SPR_Y);
	FaceSetNo(FACE_SPR_TILE_NO_NORMAL);
}
//---------------------------------------------------------------------------
EWRAM_CODE void FaceInitSpr()
{
	u8* pSprPal    = GBFSGetFilePointer("spr.pal");
	u8* pSprDat    = GBFSGetFilePointer("spr.img");
	u32 SprDatSize = GBFSGetFileSize();

	if(pSprPal == NULL || pSprDat == NULL)
	{
		SystemError("[Err] Sprite File Not Found.", NULL);
	}

	SpriteSetPalData16((u16*)pSprPal);
	SpriteSetData((u16*)pSprDat, SprDatSize);

	SpriteSetSize(FACE_SPR_NO, SP_SIZE_64, SP_SQUARE, SP_COLOR_16);
}
//---------------------------------------------------------------------------
EWRAM_CODE void FaceSetNo(u16 tileNo)
{
	SpriteSetChr(FACE_SPR_NO, tileNo);
}
//---------------------------------------------------------------------------
EWRAM_CODE void FaceSetX(u16 x)
{
	Face.x = x;
	SpriteMove(FACE_SPR_NO, Face.x, Face.y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void FaceSetY(u16 y)
{
	Face.y = y;
	SpriteMove(FACE_SPR_NO, Face.x, Face.y);
}
