
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "ad_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "rand.h"
#include "music.h"
#include "anime.h"
#include "main.h"

//#include "header\font\BGFont.h"


//Mode3
//bg wi
//Spr 

//---------------------------------------------------------------------------
u8 State;

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:    StateInit();    break;
			case STATE_RESET:   StateReset();   break;
			case STATE_FADEIN:  StateFadeIn();  break;
			case STATE_FADEOUT: StateFadeOut(); break;
			case STATE_KEY:     StateKey();     break;
			case STATE_ANIME:   StateAnime();   break;

			default:
				DebugMessage("[Err] main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitLib();
	InitKey();
	InitIRQ();
	InitFont();
	InitSprite();

	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);
	InitGBFS();

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitLib()
{
//	LibInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER2;

	REG_TM2D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM2CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitFont()
{
	ST_FONT_SRC half;		//p

	half.pDat   = (u8*)&mplus_s10rBitmap;
	half.pSheet = (u16*)&mplus_sfnt_txt;
	half.imgCx  = MPLUS_S10R_IMG_CX;
	half.cnt    = MPLUS_S10R_FONT_CNT;
	half.cx     = MPLUS_S10R_FONT_CX;
	half.cy     = MPLUS_S10R_FONT_CY;

	Mode3SetFont(NULL, &half);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitGBFS()
{
	if( GBFSInit() == FALSE )
	{
		SystemError("[Err] GBFS Not Found.", NULL);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	Mode3DrawBlend(BLEND_MODE_DARK | BLEND_TOP_OBJ, 0, 15);
	Mode3DrawFadeIn(0);

	MusicInit();
	AnimeInit();
	FaceInit();

	AnimeSetNo(ANIME_NO_NORMAL);
	AnimeExec();

	State = STATE_FADEOUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateFadeIn()
{
	Mode3DrawBlend(BLEND_MODE_DARK | BLEND_TOP_OBJ, 0, 15);
	Mode3DrawFadeIn(20);

	State = STATE_FADEOUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateFadeOut()
{
	Mode3DrawFadeOut(20);

	State = STATE_KEY;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateKey()
{
	u16 key;

Restart:

	for(;;)
	{
		WaitForVsync();
		key = KeyGet();

		if(key != KEY_NONE)
		{
			break;
		}
	}

	if(key & KEY_START)
	{
		MusicStart();
		goto Restart;
	}

	if(key & KEY_SELECT)
	{
		MusicStop();
		goto Restart;
	}

	if(key & KEY_A)
	{
		AnimeSetNo(ANIME_NO_SHAKE);
	}
	else if(key & KEY_B)
	{
		AnimeSetNo(ANIME_NO_AWAKE);
	}
	else if(key & KEY_L)
	{
		AnimeSetNo(ANIME_NO_SLEEP1);
	}
	else if(key & KEY_R)
	{
		AnimeSetNo(ANIME_NO_SLEEP2);
	}
	else if(key & KEY_UP)
	{
		AnimeSetNo(ANIME_NO_FACE_AWAKE);
	}
	else if(key & KEY_DOWN)
	{
		AnimeSetNo(ANIME_NO_DROP);
	}
	else if(key & KEY_RIGHT)
	{
		AnimeSetNo(ANIME_NO_FACE_NORMAL);
	}
	else if(key & KEY_LEFT)
	{
		AnimeSetNo(ANIME_NO_FACE_SCARE);
	}

	State = STATE_ANIME;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateAnime()
{
	AnimeExec();

	State = STATE_KEY;
}




//---------------------------------------------------------------------------
EWRAM_CODE void SystemError(char* msg1, char* msg2)
{
	LibMode3SetScreenBuffer((u16*)VideoBuffer);

	if(msg1 != NULL) Mode3DrawFontStr(0,  0, (u8*)msg1, COLOR_GRAY);
	if(msg2 != NULL) Mode3DrawFontStr(0, 11, (u8*)msg2, COLOR_GRAY);

	for(;;){}
}
